/* DirectFB presentation app
 *
 * Copyright (C) 2001  convergence integrated media
 * Authors: Sven Neumann <sven@convergence.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <directfb.h>
#include <glib-object.h>

#include "apptypes.h"

#include "item.h"
#include "context.h"


static void  context_class_init (ContextClass *klass);
static void  context_init       (Context      *context);
static void  context_finalize   (GObject     *object);


static guchar black[4] = { 0, 0, 0, 0xFF };
static GObjectClass *parent_class = NULL;


GType
context_get_type (void)
{
  static GType context_type = 0;

  if (!context_type)
    {
      static const GTypeInfo context_info =
      {
        sizeof (ContextClass),
	(GBaseInitFunc) NULL,
	(GBaseFinalizeFunc) NULL,
	(GClassInitFunc) context_class_init,
	NULL,		/* class_finalize */
	NULL,		/* class_data     */
	sizeof (Context),
	0,              /* n_preallocs    */
	(GInstanceInitFunc) context_init,
      };

      context_type = 
        g_type_register_static (G_TYPE_OBJECT, "Context", &context_info, 0);
    }
  
  return context_type;
}

static void
context_class_init (ContextClass *klass)
{
  GObjectClass *object_class;

  object_class = G_OBJECT_CLASS (klass);
  parent_class = g_type_class_peek_parent (klass);

  object_class->finalize = context_finalize;
}

static void
context_init (Context *context)
{
  context->surface  = NULL;
  context->w        = 0;
  context->h        = 0;

  context->surfaces = NULL;
  context->fgcolors = NULL;
  context->bgcolors = NULL;
  context->faces    = NULL;
  context->bgimages = NULL;
  context->bullets  = NULL;
  context->sizes    = NULL;
  context->widths   = NULL;
  context->heights  = NULL;
  context->xs       = NULL;
  context->ys       = NULL;
  context->effects  = NULL;
}

Context * 
context_new (void)
{
  return CONTEXT (g_object_new (TYPE_CONTEXT, NULL));
}

static void
context_finalize (GObject *object)
{
  Context *context = CONTEXT (object);

  g_slist_free (context->surfaces);
  g_slist_free (context->fgcolors);
  g_slist_free (context->bgcolors);
  g_slist_free (context->faces);
  g_slist_free (context->bgimages);
  g_slist_free (context->bullets);
  g_slist_free (context->sizes);
  g_slist_free (context->widths);
  g_slist_free (context->heights);
  g_slist_free (context->xs);
  g_slist_free (context->ys);
  g_slist_free (context->effects);

  if (parent_class->finalize)
    parent_class->finalize (object);
}

void
context_push (Context *context, 
              Item    *item)
{
  g_return_if_fail (context != NULL);
  g_return_if_fail (item != NULL);
  
  if (item->flags & FGCOLOR)
    context->fgcolors = g_slist_prepend (context->fgcolors, 
                                         (gpointer) item->fgcolor);

  if (item->flags & BGCOLOR)
    context->bgcolors = g_slist_prepend (context->bgcolors, 
                                         (gpointer) item->bgcolor);

  if (item->flags & FACE)
    context->faces = g_slist_prepend (context->faces, 
                                      (gpointer) item->face);

  if (item->flags & BGIMAGE)
    context->bgimages = g_slist_prepend (context->bgimages, 
                                         (gpointer) item->bgimage);
  
  if (item->flags & BULLET)
    context->bullets = g_slist_prepend (context->bullets, 
                                        (gpointer) item->bullet);

  if (item->flags & SIZE)
    context->sizes = g_slist_prepend (context->sizes, 
                                      (gpointer) item->size);

  if (item->flags & WIDTH)
    context->widths = g_slist_prepend (context->widths, 
                                       (gpointer) item->width);

  if (item->flags & HEIGHT)
    context->heights = g_slist_prepend (context->heights, 
                                       (gpointer) item->height);

  if (item->flags & XOFFSET)
    context->xs = g_slist_prepend (context->xs, 
                                   (gpointer) item->x);

  if (item->flags & YOFFSET)
    context->ys = g_slist_prepend (context->ys, 
                                   (gpointer) item->y);

  if (item->flags & EFFECT)
    context->effects = g_slist_prepend (context->effects, 
                                        (gpointer) item->effect);
}

void
context_pop (Context *context, 
             Item    *item)
{
  g_return_if_fail (context != NULL);
  g_return_if_fail (item != NULL);

  /*  FIXME:  shouldn't use g_slist_remove here, just pop  */

  if (item->flags & FGCOLOR)
    context->fgcolors = g_slist_remove (context->fgcolors,  
                                        (gpointer) item->fgcolor);

  if (item->flags & BGCOLOR)
    context->bgcolors = g_slist_remove (context->bgcolors, 
                                        (gpointer) item->bgcolor);

  if (item->flags & FACE)
    context->faces = g_slist_remove (context->faces, 
                                     (gpointer) item->face);

  if (item->flags & BGIMAGE)
    context->bgimages = g_slist_remove (context->bgimages, 
                                        (gpointer) item->bgimage);
  
  if (item->flags & BULLET)
    context->bullets = g_slist_remove (context->bullets, 
                                       (gpointer) item->bullet);

  if (item->flags & SIZE)
    context->sizes = g_slist_remove (context->sizes, 
                                     (gpointer) item->size);  

  if (item->flags & WIDTH)
    context->widths = g_slist_remove (context->widths, 
                                      (gpointer) item->width);
  
  if (item->flags & HEIGHT)
    context->heights = g_slist_remove (context->heights, 
                                       (gpointer) item->height);
  
  if (item->flags & XOFFSET)
    context->xs = g_slist_remove (context->xs, 
                                  (gpointer) item->x);

  if (item->flags & YOFFSET)
    context->ys = g_slist_remove (context->ys, 
                                  (gpointer) item->y);

 if (item->flags & EFFECT)
    context->effects = g_slist_remove (context->effects, 
                                       (gpointer) item->effect);
}

void
context_push_surface (Context          *context,
                      IDirectFBSurface *surface)
{
  g_return_if_fail (context != NULL);
  g_return_if_fail (surface != NULL);
 
  context->surfaces = g_slist_prepend (context->surfaces, surface);
  context->surface = surface;
  surface->GetSize (surface, &context->w, &context->h);
}

void
context_pop_surface (Context *context)
{
  GSList *next;

  g_return_if_fail (context != NULL);
 
  next = g_slist_next (context->surfaces);
  g_slist_free_1 (context->surfaces);
  context->surfaces = next;
  
  if (next)
    context->surface = (IDirectFBSurface *) next->data;
}

guchar * 
context_get_fgcolor (Context *context)
{
  g_return_val_if_fail (context != NULL, NULL);

  return context->fgcolors ? (guchar *) context->fgcolors->data : black;
}

guchar * 
context_get_bgcolor (Context *context)
{
  g_return_val_if_fail (context != NULL, NULL);

  return context->bgcolors ? (guchar *) context->bgcolors->data : black;
}

gchar * 
context_get_face (Context *context)
{
  g_return_val_if_fail (context != NULL, NULL);
  
  return context->faces ? (gchar *) context->faces->data : NULL;
}

gchar * 
context_get_bgimage (Context *context)
{
  g_return_val_if_fail (context != NULL, NULL);
  
  return context->bgimages ? (gchar *) context->bgimages->data : NULL;
}

gchar * 
context_get_bullet (Context *context)
{
  g_return_val_if_fail (context != NULL, NULL);
  
  return context->bullets ? (gchar *) context->bullets->data : NULL;
}

gint 
context_get_size (Context *context)
{
  g_return_val_if_fail (context != NULL, 0);
  
  return context->sizes ? (gint) context->sizes->data : 0;
}
 
void
context_get_rect (Context      *context,
                  DFBRectangle *rect)
{
  g_return_if_fail (context != NULL);
  g_return_if_fail (rect != NULL);
  
  rect->x = rect->y = 0;
  rect->w = context->w;
  rect->h = context->h;

  rect->w = context->widths  ? (gint) context->widths->data  : context->w;
  rect->h = context->heights ? (gint) context->heights->data : context->h;
  rect->x = context->xs ? (gint) context->xs->data : 0;
  rect->y = context->ys ? (gint) context->ys->data : 0;
}

EffectFlags
context_get_effect (Context *context)
{
  g_return_val_if_fail (context != NULL, EFFECT_NONE);
 
  return context->effects ? (EffectFlags) context->effects->data : EFFECT_NONE;
}
