/* DirectFB presentation app
 *
 * Copyright (C) 2001  convergence integrated media
 * Authors: Sven Neumann <sven@convergence.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <stdio.h>
#include <stdlib.h>

#include <directfb.h>
#include <glib-object.h>

#include "apptypes.h"

#include "item.h"
#include "parser.h"
#include "resources.h"


static void  item_class_init (ItemClass *klass);
static void  item_init       (Item      *item);
static void  item_finalize   (GObject   *object);


static GObjectClass *parent_class = NULL;


GType
item_get_type (void)
{
  static GType item_type = 0;

  if (!item_type)
    {
      static const GTypeInfo item_info =
      {
        sizeof (ItemClass),
	(GBaseInitFunc) NULL,
	(GBaseFinalizeFunc) NULL,
	(GClassInitFunc) item_class_init,
	NULL,		/* class_finalize */
	NULL,		/* class_data     */
	sizeof (Item),
	0,              /* n_preallocs    */
	(GInstanceInitFunc) item_init,
      };

      item_type = 
        g_type_register_static (G_TYPE_OBJECT, "item", &item_info, 0);
    }
  
  return item_type;
}

static void
item_class_init (ItemClass *klass)
{
  GObjectClass *object_class;

  object_class = G_OBJECT_CLASS (klass);
  parent_class = g_type_class_peek_parent (klass);

  klass->prepare          = NULL;
  klass->realize          = NULL;
  klass->unrealize        = NULL;
  klass->parse_attributes = NULL;

  object_class->finalize = item_finalize;
}

static void
item_init (Item *item)
{
  item->value   = NULL;
  item->face    = NULL;
  item->bgimage = NULL;
  item->bullet  = NULL;
  item->flags   = 0;
}

Item *
item_new (const gchar *name)
{
  GObject *object;
  GType    type;

  g_return_val_if_fail (name != NULL, NULL);

  type = g_type_from_name (name);

  if (type == 0)
    return NULL;

  object = g_object_new (type, NULL);

  if (!IS_ITEM (object))
    {
      g_object_unref (object);
      return FALSE;
    }

  return ITEM (object);
}

static void
item_finalize (GObject *object)
{
  Item *item = ITEM (object);

  g_free (item->value);
  g_free (item->face);
  g_free (item->bgimage);
  g_free (item->bullet);

  if (parent_class->finalize)
    parent_class->finalize (object);
}

gboolean
item_prepare (Item *item)
{
  ItemClass *klass;

  g_return_val_if_fail (IS_ITEM (item), FALSE);

  if (item->flags & FACE &&
      resources_get_font (item->face, 18) == NULL)
    return FALSE;

  if (item->flags & BGIMAGE &&
      resources_get_image_provider (item->bgimage) == NULL)
    return FALSE;

  if (item->flags & BULLET &&
      resources_get_image_provider (item->bullet) == NULL)
    return FALSE;

  klass = ITEM_CLASS (G_OBJECT_GET_CLASS (item));
  if (klass->prepare)
    return klass->prepare (item);

  return TRUE;
}

void       
item_realize (Item *item)
{
  ItemClass *klass;

  g_return_if_fail (IS_ITEM (item));

  if (item->flags & BGIMAGE)
    resources_get_surface (item->bgimage);

  if (item->flags & BULLET)
    resources_get_surface (item->bullet);

  klass = ITEM_CLASS (G_OBJECT_GET_CLASS (item));
  if (klass->realize)
    klass->realize (item);
}

void       
item_unrealize (Item *item)
{
  ItemClass *klass;

  g_return_if_fail (IS_ITEM (item));

  if (item->flags & BGIMAGE)
    resources_release_surface (item->bgimage);

  if (item->flags & BULLET)
    resources_release_surface (item->bullet);

  klass = ITEM_CLASS (G_OBJECT_GET_CLASS (item));
  if (klass->unrealize)
    klass->unrealize (item);
}

void
item_parse_attributes (Item         *item,
                       const gchar **names,
                       const gchar **values)
{
  ItemClass   *klass;
  const gchar *str;
  
  g_return_if_fail (IS_ITEM (item));

  str = parse_attributes (names, values, "bgcolor");
  if (parse_color (str, item->bgcolor))
    item->flags |= BGCOLOR;

  str = parse_attributes (names, values, "fgcolor");
  if (parse_color (str, item->fgcolor))
    item->flags |= FGCOLOR;

  str = parse_attributes (names, values, "face");
  if (str)
    {
      g_free (item->face);
      item->face = g_strdup (str);
      item->flags |= FACE;
    }

  str = parse_attributes (names, values, "bgimage");
  if (str)
    {
      g_free (item->bgimage);
      item->bgimage = g_strdup (str);
      item->flags |= BGIMAGE;
    }

  str = parse_attributes (names, values, "bullet");
  if (str)
    {
      g_free (item->bullet);
      item->bullet = g_strdup (str);
      item->flags |= BULLET;
    }

  str = parse_attributes (names, values, "size");
  if (parse_number (str, &item->size))
    item->flags |= SIZE;

  str = parse_attributes (names, values, "width");
  if (parse_number (str, &item->width))
    item->flags |= WIDTH;

  str = parse_attributes (names, values, "height");
  if (parse_number (str, &item->height))
    item->flags |= HEIGHT;

  str = parse_attributes (names, values, "x");
  if (parse_number (str, &item->x))
    item->flags |= XOFFSET;

  str = parse_attributes (names, values, "y");
  if (parse_number (str, &item->y))
    item->flags |= YOFFSET;

  str = parse_attributes (names, values, "effect");
  if (parse_effect (str, &item->effect))
    item->flags |= EFFECT;

  klass = ITEM_CLASS (G_OBJECT_GET_CLASS (item));
  if (klass->parse_attributes)
    klass->parse_attributes (item, names, values);
}

void
item_set_value (Item        *item,
                const gchar *value)
{
  g_return_if_fail (IS_ITEM (item));

  g_free (item->value);
  item->value = g_strdup (value);
}

