/* DirectFB presentation app
 *
 * Copyright (C) 2001  convergence integrated media
 * Authors: Sven Neumann <sven@convergence.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <directfb.h>

#include <glib-object.h>

#include "apptypes.h"

#include "item.h"
#include "parser.h"
#include "slide.h"
#include "slides.h"


#define BUF_SIZE  8192


typedef enum 
{
  PARSER_START,
  PARSER_IN_SLIDES,
  PARSER_IN_SLIDE,
  PARSER_IN_ITEM,
  PARSER_FINISH,
  PARSER_UNKNOWN
} ParserState;

typedef struct _Parser Parser;
struct _Parser
{
  ParserState  state;
  ParserState  last_state;
  gint         unknown_depth;

  GString     *value;

  Slides      *slides;
  Slide       *slide;
  Item        *item;
};


static gchar * parse_encoding     (const gchar *string,
                                   gint         len);

static void  parser_start_element (GMarkupParseContext  *context,
                                   const gchar          *element_name,
                                   const gchar         **attribute_names,
                                   const gchar         **attribute_values,
                                   gpointer              user_data,
                                   GError              **error);
static void  parser_end_element   (GMarkupParseContext  *context,
                                   const gchar          *element_name,
                                   gpointer              user_data,
                                   GError              **error);
static void  parser_text          (GMarkupParseContext  *context,
                                   const gchar          *text,
                                   gsize                 text_len,
                                   gpointer              user_data,
                                   GError              **error);
static void  parser_start_unknown (Parser               *parser);         
static void  parser_end_unknown   (Parser               *parser);         
static void  parser_error         (GMarkupParseContext  *context,
                                   GError               *error,
                                   gpointer              user_data);


static const GMarkupParser markup_parser =
{
  parser_start_element,
  parser_end_element,
  parser_text,
  NULL,  /*  passthrough  */
  parser_error
};


Slides *
parse (const gchar *filename)
{
  GMarkupParseContext *context;
  Parser              *parser;
  gchar     buffer[BUF_SIZE];
  gchar    *dst;
  gchar    *encoding;
  guint     size;
  guint     bytes;
  FILE     *fp;
  GError   *error;
  gboolean  success = TRUE;

  g_return_val_if_fail (filename != NULL, FALSE);

  if ((fp = fopen (filename, "r")) ==  NULL)
    {
      g_printerr ("Couldn't open file '%s': %s\n", 
                  filename, g_strerror (errno));
      return NULL;
    }

  bytes = fread (buffer, sizeof (gchar), sizeof (buffer), fp);
  encoding = parse_encoding (buffer, bytes);

  if (encoding)
    {
      if (g_ascii_strcasecmp (encoding, "utf8")  == 0 || 
          g_ascii_strcasecmp (encoding, "utf-8") == 0)
        {
          g_free (encoding);
          encoding = NULL;
        }
      else
        {
          g_print ("XML file claims to be %s encoded, converting to UTF-8.\n",
                   encoding);
        }
    }

  parser = g_new0 (Parser, 1);
  parser->value = g_string_new (NULL);

  context = g_markup_parse_context_new (&markup_parser,
                                        0,      /*  parser flags       */ 
                                        parser, /*  user_data          */
                                        NULL    /*  user_data_dnotify  */);

  size = BUF_SIZE;
  dst  = buffer;

  do 
    {
      if (encoding)
        {
          gchar *converted;
          gint   converted_bytes;
          gint   missed;

          converted = g_convert (buffer, bytes, "UTF-8", encoding, 
                                 NULL, &converted_bytes, &error);
          if (converted)
            {
              success = g_markup_parse_context_parse (context, converted, 
                                                      converted_bytes, &error);
              g_free (converted);
              
              missed = bytes - converted_bytes;
              if (missed > 0)
                {
                  memmove (buffer, buffer + converted_bytes, missed);
                  dst = buffer + missed;
                  size -= missed;
                }
            }
          else
            {
              g_printerr ("Conversion failed: %s\n", error->message);
              success = FALSE;
              g_clear_error (&error);
            }
        }
      else
        {
          success = g_markup_parse_context_parse (context,
                                                  buffer, bytes, &error);
        }
    }
  while (success &&
	 (bytes = fread (dst, sizeof (gchar), size, fp)) > 0);

  if (!success || !g_markup_parse_context_end_parse (context, &error))
    g_clear_error (&error);

  g_markup_parse_context_free (context);

  fclose (fp);

  g_string_free (parser->value, TRUE);

  if (parser->state == PARSER_FINISH)
    return parser->slides;

  if (parser->slides)
    g_object_unref (parser->slides);
  
  return NULL;
}

static void
parser_start_element (GMarkupParseContext  *context,
                      const gchar          *element_name,
                      const gchar         **attribute_names,
                      const gchar         **attribute_values,
                      gpointer              user_data,
                      GError              **error)
{
  Parser   *parser = (Parser *) user_data;
  gpointer  item   = NULL;

  switch (parser->state)
    {
    case PARSER_START:
      if (strcmp (element_name, "slides"))
        parser_start_unknown (parser);
      else
	{
          parser->slides = item = item_new (element_name);
	  parser->state = PARSER_IN_SLIDES;
	}
      break;

    case PARSER_IN_SLIDES:
      if (strcmp (element_name, "slide"))
        parser_start_unknown (parser);
      else
	{
          parser->slide = item = item_new (element_name);
	  parser->state = PARSER_IN_SLIDE;
	}
      break;
      
    case PARSER_IN_SLIDE:
      parser->item = item = item_new (element_name);
      if (item)
        parser->state = PARSER_IN_ITEM;
      else
        parser_start_unknown (parser);
      break;
      
    case PARSER_IN_ITEM:
    case PARSER_FINISH:
    case PARSER_UNKNOWN:
      parser_start_unknown (parser);
    }

  if (item)
    item_parse_attributes (ITEM (item), attribute_names, attribute_values);
}

static void  
parser_end_element (GMarkupParseContext  *context,
                    const gchar          *element_name,
                    gpointer              user_data,
                    GError              **error)
{
  Parser *parser = (Parser *) user_data;

  switch (parser->state)
    {
    case PARSER_START:
      g_warning ("I'm dazed and confused.\n");
      break;

    case PARSER_IN_SLIDES:
      parser->state = PARSER_FINISH;
      break;

    case PARSER_IN_SLIDE:
      parser->state = PARSER_IN_SLIDES;
      slides_append_slide (parser->slides, parser->slide);
      parser->slide = NULL;
      break;
   
    case PARSER_IN_ITEM:
      parser->state = PARSER_IN_SLIDE;
      item_set_value (parser->item, g_strstrip (parser->value->str));
      slide_append_item (parser->slide, parser->item);
      parser->item = NULL;
     break;

    case PARSER_FINISH:
    case PARSER_UNKNOWN:
      parser_end_unknown (parser);
    }

  g_string_assign (parser->value, "");
}

static void
parser_text (GMarkupParseContext  *context,
             const gchar          *text,
             gsize                 text_len,
             gpointer              user_data,
             GError              **error)

{
  Parser *parser = (Parser *) user_data;

  g_string_append_len (parser->value, text, text_len);
}

static void
parser_start_unknown (Parser *parser)
{
  if (parser->unknown_depth == 0)
    {
      parser->last_state = parser->state;
      parser->state = PARSER_UNKNOWN;
    }
  parser->unknown_depth++;
}

static void
parser_end_unknown (Parser *parser)
{
  parser->unknown_depth--;
  if (parser->unknown_depth == 0)
    parser->state = parser->last_state;
}

static void         
parser_error (GMarkupParseContext *context,
              GError              *error,
              gpointer             user_data)
{
  g_printerr ("Fatal parse error: %s.\n", error->message);
}

const gchar *
parse_attributes (const gchar **attribute_names,
                  const gchar **attribute_values,
                  const gchar  *name)
{
  gint i;
  
  if (attribute_names && attribute_values)
    {
      for (i = 0; attribute_names[i] && attribute_values[i]; i++)
	{
	  if (strcmp (attribute_names[i], name) == 0)
	    return attribute_values[i];
	}
    }

  return NULL;
}

gboolean
parse_color (const gchar *str, 
             guchar      *color)
{
  guint red, green, blue, alpha;
  gint  num;

  if (str && *str && *str == '#')
    {
      num = sscanf (str + 1, "%2x%2x%2x%2x", &red, &green, &blue, &alpha);
  
      switch (num)
        {
          case 3:
            color[0] = red;
            color[1] = green;
            color[2] = blue;
            color[3] = 0xFF;
            return TRUE;

          case 4:
            color[0] = red;
            color[1] = green;
            color[2] = blue;
            color[3] = alpha;
            return TRUE;

          default:
            break;
        }
    }
  
  return FALSE;
}

gboolean
parse_number (const gchar *str, 
              gint        *number)
{
  gint   result;
  gchar *end;

  if (str && *str)
    {
      result = strtol (str, &end, 0);
      if (end && *end == '\0')
        {
          *number = result;
          return TRUE;
        }
    }

  return FALSE;
}
      
gboolean 
parse_effect (const gchar *str, 
              EffectFlags *flags)
{
  if (str && *str)
    {
      *flags = EFFECT_NONE;

      if (strstr (str, "blend"))
        *flags |= EFFECT_BLEND;

      if (strstr (str, "slide"))
        *flags |= EFFECT_SLIDE;

      if (strstr (str, "left"))
        *flags |= EFFECT_LEFT;
      if (strstr (str, "right"))
        *flags |= EFFECT_RIGHT;
      if (strstr (str, "top"))
        *flags |= EFFECT_TOP;
      if (strstr (str, "bottom"))
        *flags |= EFFECT_BOTTOM;
    
      return TRUE;
    }

  return FALSE;
}

static gchar * 
parse_encoding (const gchar *string,
                gint         len)
{
  gchar *start;
  gchar *end;

  if (len < 20 || !string)
    return NULL;

  start = g_strstr_len (string, len, "<?xml");
  if (!start)
    return NULL;

  end = g_strstr_len (start, len - (start - string), "?>");
  if (!end)
    return NULL;
  
  len = end - start;
  if (len < 13)
    return NULL;

  start = g_strstr_len (start + 1, len - 1, "encoding=");
  if (!start)
    return NULL;

  start += 9;
  if (*start != '\"' && *start != '\'')
    return NULL;
  
  len = end - start;
  if (len < 1)
    return NULL;

  end = memchr (start + 1, *start, len - 1);

  if (!end)
    return NULL;

  len = end - start - 1;
  if (len < 1)
    return NULL;

  return g_strndup (start + 1, len);
}
