/* DirectFB presentation app
 *
 * Copyright (C) 2001  convergence integrated media
 * Authors: Sven Neumann <sven@convergence.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <stdlib.h>
#include <string.h>

#include <directfb.h>
#include <glib.h>

#include "dfbpoint.h"
#include "resources.h"


typedef enum
{
  RESOURCE_IMAGE,
  RESOURCE_VIDEO,
  RESOURCE_SURFACE,
  RESOURCE_FONT
} ResourceType;

typedef struct _Resource Resource;
struct _Resource
{
  ResourceType type;
  union
  {
    IDirectFBImageProvider *image_provider;
    IDirectFBVideoProvider *video_provider;
    IDirectFBSurface       *surface;
    IDirectFBFont          *font;
  } data;
};

typedef struct _ResourceKey ResourceKey;
struct _ResourceKey
{
  ResourceType  type;
  const gchar  *name;
  gint          size;
};


static Resource    * resources_lookup   (ResourceType       type,
                                         const gchar       *name,
                                         gint               size);
static gpointer      resource_new       (const ResourceKey *key);
static void          resource_free      (Resource          *resource);
static guint         resource_hash      (const Resource    *resource);
static ResourceKey * resource_key_dup   (const ResourceKey *key);
static void          resource_key_free  (ResourceKey       *key);
static guint         resource_key_hash  (const ResourceKey *key);
static gboolean      resource_key_equal (const ResourceKey *key1,
                                         const ResourceKey *key2);
static void          resource_destroy   (gpointer           key,
                                         gpointer           value,
                                         gpointer           data);

static DFBResult     resource_surface_create (const gchar       *name,
                                              IDirectFBSurface **surface);


static GCache *cache = NULL;


IDirectFBImageProvider * 
resources_get_image_provider (const gchar *name)
{
  Resource *resource;

  resource = resources_lookup (RESOURCE_IMAGE, name, 0);
  
  return resource->data.image_provider;
}

IDirectFBVideoProvider * 
resources_get_video_provider (const gchar *name)
{
  Resource *resource;

  resource = resources_lookup (RESOURCE_VIDEO, name, 0);

  return resource->data.video_provider;
}

IDirectFBSurface * 
resources_get_surface (const gchar *name)
{
  Resource *resource;

  resource = resources_lookup (RESOURCE_SURFACE, name, 0);

  return resource->data.surface;
}

IDirectFBFont * 
resources_get_font (const gchar *name,
                    gint         size)
{
  Resource *resource;

  resource = resources_lookup (RESOURCE_FONT, name, size);
  
  return resource->data.font;
}

static Resource *
resources_lookup (ResourceType  type,
                  const gchar  *name,
                  gint          size)
{
  ResourceKey key;

  g_return_val_if_fail (name != NULL, NULL);

  key.type = type;
  key.name = name;
  key.size = size;

  if (!cache)
    cache = g_cache_new ((GCacheNewFunc)     resource_new, 
                         (GCacheDestroyFunc) resource_free,
                         (GCacheDupFunc)     resource_key_dup, 
                         (GCacheDestroyFunc) resource_key_free,
                         (GHashFunc)         resource_key_hash, 
                         (GHashFunc)         resource_hash, 
                         (GEqualFunc)        resource_key_equal);
  
  return g_cache_insert (cache, &key);
}

void
resources_release_surface (const gchar *name)
{
  Resource *resource;

  if (!cache)
    return;

  resource = resources_lookup (RESOURCE_SURFACE, name, 0);
  if (resource)
    {
      /* this looks wrong, but it is correct, resources_lookup increases
         the ref_count, so we need to remove it from the cache twice */
      g_cache_remove (cache, (gpointer) resource);
      g_cache_remove (cache, (gpointer) resource);
    }
}

void
resources_free (void)
{
  if (cache)
    {
      g_cache_value_foreach (cache, resource_destroy, NULL);
      g_cache_destroy (cache);
    }
}

static ResourceKey *
resource_key_dup (const ResourceKey *key)
{
  ResourceKey *dup;

  dup = g_new (ResourceKey, 1);

  dup->type = key->type;
  dup->name = g_strdup (key->name);
  dup->size = key->size;

  return dup;
}

static guint
resource_key_hash (const ResourceKey *key)
{
  return g_str_hash (key->name) + key->type + key->size;
}

static gboolean
resource_key_equal (const ResourceKey *key1,
                    const ResourceKey *key2)
{
  return (key1->type == key2->type &&
          key1->size == key2->size &&
          strcmp (key1->name, key2->name) == 0);
}

static void
resource_key_free (ResourceKey *key)
{
  g_free ((gchar *) key->name);
  g_free (key);
}

static gpointer
resource_new (const ResourceKey *key)
{
  Resource    *resource;
  gchar       *filename;
  gchar       *name;
  const gchar *msg   = NULL;
  GError      *error = NULL;
  DFBResult    ret   = DFB_OK;

  resource = g_new0 (Resource, 1);
  resource->type = key->type;

  name = g_filename_from_utf8 (key->name, -1, NULL, NULL, &error);

  if (!name)
    {
      g_get_charset (&msg);
      g_printerr ("Couldn't convert filename '%s' to your filesystem encoding (%s): %s\n"
                  "You might need to unset the environment variable G_BROKEN_FILENAMES or set CHARSET to a correct value.\n",
                  key->name, msg, error->message);
      g_error_free (error);

      return resource;
    }

  if (g_path_is_absolute (name))
    {
      filename = name;
    }
  else
    {
      filename = g_build_filename (document_root, name, NULL);
      g_free (name);
    }

  switch (resource->type)
    {
    case RESOURCE_IMAGE:
      msg = "Couldn't load image '%s': %s\n";
      ret = dfb->CreateImageProvider (dfb, filename, 
                                      &resource->data.image_provider);
      break;

    case RESOURCE_VIDEO:
      msg = "Couldn't load video '%s': %s\n";
      ret = dfb->CreateVideoProvider (dfb, filename, 
                                      &resource->data.video_provider);
      break;
      
    case RESOURCE_SURFACE:
      msg = "Couldn't create surface for '%s': %s\n";
      ret = resource_surface_create (key->name, &resource->data.surface);
      break;

    case RESOURCE_FONT:
      {
        DFBFontDescription  dsc;
        
        msg = "Couldn't load font '%s': %s\n";

        dsc.flags  = DFDESC_HEIGHT;
        dsc.height = key->size;

        ret = dfb->CreateFont (dfb, filename, &dsc, &resource->data.font);
      }
      break;

    default:
      g_assert_not_reached ();
      break;
    }
  
  if (ret != DFB_OK)
    {
      resource->data.image_provider = NULL;

      name = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
      g_printerr (msg, name, DirectFBErrorString (ret));
      g_free (name);
    }

  g_free (filename);

  return resource;
}

static void
resource_free (Resource *resource)
{
  resource_destroy (NULL, resource, NULL);
  g_free (resource);
}

static void
resource_destroy (gpointer key,
                  gpointer value,
                  gpointer data)
{
  Resource *resource = (Resource *) value;

  switch (resource->type)
    {
    case RESOURCE_IMAGE:
      resource->data.image_provider->Release (resource->data.image_provider);
      break;
      
    case RESOURCE_SURFACE:
      resource->data.surface->Release (resource->data.surface);
      break;

    case RESOURCE_VIDEO:
      resource->data.video_provider->Release (resource->data.video_provider);
      break;

    case RESOURCE_FONT:
      resource->data.font->Release (resource->data.font);
      break;
    }
}

static guint
resource_hash (const Resource *resource)
{
  return g_direct_hash (resource->data.font);
}

static DFBResult
resource_surface_create (const gchar       *name,
                         IDirectFBSurface **surface)
{
  DFBResult               result;
  IDirectFBImageProvider *provider;
  DFBSurfaceDescription   dsc;

  provider = resources_get_image_provider (name);
  if (!provider)
    return DFB_FAILURE;

  provider->GetSurfaceDescription (provider, &dsc);

  result = dfb->CreateSurface (dfb, &dsc, surface);

  if (result == DFB_OK)
    provider->RenderTo (provider, *surface, NULL);

  return result;
}
