/* DirectFB presentation app
 *
 * Copyright (C) 2001  convergence integrated media
 * Authors: Sven Neumann <sven@convergence.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <sys/time.h>

#include <directfb.h>
#include <glib-object.h>

#include "apptypes.h"

#include "action.h"
#include "context.h"
#include "dfbpoint.h"
#include "image.h"
#include "render.h"
#include "slide.h"
#include "slide-view.h"
#include "text.h"
#include "video.h"


static void  slide_view_class_init (SlideViewClass *klass);
static void  slide_view_init       (SlideView      *view);
static void  slide_view_finalize   (GObject        *object);

static IDirectFBWindow * slide_view_window_create (gint        *width,
                                                   gint        *height);
static void              slide_view_render_slide  (SlideView   *view,
                                                   Context     *context);
static void              slide_view_start         (SlideView   *view);
static void              slide_view_stop          (SlideView   *view);
static void              slide_view_effect        (SlideView   *view,
                                                   EffectFlags  effect);


static GObjectClass *parent_class = NULL;


GType
slide_view_get_type (void)
{
  static GType slide_view_type = 0;

  if (!slide_view_type)
    {
      static const GTypeInfo slide_view_info =
      {
        sizeof (SlideViewClass),
	(GBaseInitFunc) NULL,
	(GBaseFinalizeFunc) NULL,
	(GClassInitFunc) slide_view_class_init,
	NULL,		/* class_finalize */
	NULL,		/* class_data     */
	sizeof (SlideView),
	0,              /* n_preallocs    */
	(GInstanceInitFunc) slide_view_init,
      };

      slide_view_type = 
        g_type_register_static (G_TYPE_OBJECT, 
                                "SlideView", &slide_view_info, 0);
    }
  
  return slide_view_type;
}

static void
slide_view_class_init (SlideViewClass *klass)
{
  GObjectClass *object_class;

  object_class = G_OBJECT_CLASS (klass);
  parent_class = g_type_class_peek_parent (klass);

  object_class->finalize = slide_view_finalize;
}

static void
slide_view_init (SlideView *view)
{
  view->slide      = NULL;
  view->subsurface = NULL;
  view->effect     = EFFECT_NONE;
}

SlideView *
slide_view_new (Slide   *slide,
                Context *context)
{
  SlideView *view;

  g_return_val_if_fail (IS_SLIDE (slide), NULL);
  g_return_val_if_fail (IS_CONTEXT (context), NULL);

  view = g_object_new (TYPE_SLIDE_VIEW, NULL);

  g_object_ref (slide);
  item_realize (ITEM (slide));
  view->slide = slide;

  view->window = slide_view_window_create (&view->width, &view->height);
  slide_view_render_slide (view, context);

  return view;
}

static void
slide_view_finalize (GObject *object)
{
  SlideView *view = SLIDE_VIEW (object);

  if (view->subsurface)
    view->subsurface->Release (view->subsurface);

  view->window->Release (view->window);
  item_unrealize (ITEM (view->slide));

  g_object_unref (view->slide);

  if (parent_class->finalize)
    parent_class->finalize (object);
}

void
slide_view_switch (SlideView     *from,
                   SlideView     *to,
                   ViewDirection  dir)
{
  EffectFlags effect = EFFECT_NONE;

  switch (dir)
    {
    case FORWARD:
      effect = to ? to->effect : EFFECT_NONE;
      break;
      
    case BACKWARD:
      effect = from ? from->effect : EFFECT_NONE;
      effect |= EFFECT_BACKWARD; 
    }

  if (to)
    {
      slide_view_start (to);

      if (effect & EFFECT_BACKWARD)
        {
          to->window->Lower (to->window);
          to->window->SetOpacity (to->window, 0xFF);
          slide_view_effect (from, effect);
        }
      else
        {
          slide_view_effect (to, effect);
        }

      to->window->SetOpacity (to->window, 0xFF);
      to->window->MoveTo (to->window, 0, 0);
      to->window->RaiseToTop (to->window);
    }

  if (from)
    {
      slide_view_stop (from);

      from->window->SetOpacity (from->window, 0x00);
    }
}

static void
slide_view_start (SlideView *view)
{
  GList *list;

  g_return_if_fail (IS_SLIDE_VIEW (view));

  for (list = view->slide->items; list; list = g_list_next (list))
    {
      if (IS_VIDEO (list->data))
        video_play ((Video *) list->data);
    }
}

static void
slide_view_effect (SlideView   *view,
                   EffectFlags  effect)
{
  struct timeval tv;
  gint   start_x = 0;
  gint   start_y = 0;
  gint   sign_x  = 0;
  gint   sign_y  = 0;
  gint   start_opacity = 0xFF;
  gint   sign_opacity  = 0;
  glong  start_time;
  glong  current_time;

  if (!view)
    return;

  g_return_if_fail (IS_SLIDE_VIEW (view));

  if (effect == EFFECT_NONE)
    return;

  if (effect & EFFECT_SLIDE)
    {
      if (effect & EFFECT_LEFT)
        {
          if (! (effect & EFFECT_BACKWARD))
            start_x = - view->width;
          sign_x = effect & EFFECT_BACKWARD ? -1 : 1;
        } 
      if (effect & EFFECT_RIGHT)
        {
          if (! (effect & EFFECT_BACKWARD))
            start_x = view->width;
          sign_x = effect & EFFECT_BACKWARD ? 1 : -1;
        } 
      if (effect & EFFECT_TOP)
        {
          if (! (effect & EFFECT_BACKWARD))
             start_y = - view->height;
          sign_y = effect & EFFECT_BACKWARD ? -1 : 1;
        } 
      if (effect & EFFECT_BOTTOM)
        {
          if (! (effect & EFFECT_BACKWARD))
            start_y = view->height;
          sign_y  = effect & EFFECT_BACKWARD ? 1 : -1; 
        }

      view->window->MoveTo (view->window, start_x, start_y);
    }

  if (effect & EFFECT_BLEND)
    {
      start_opacity = effect & EFFECT_BACKWARD ? 0xFF : 0;
      sign_opacity  = effect & EFFECT_BACKWARD ?   -1 : 1;
    }

  view->window->SetOpacity (view->window, start_opacity);
  view->window->RaiseToTop (view->window);

  gettimeofday (&tv, NULL);
  start_time = current_time = tv.tv_sec * 1000 + tv.tv_usec / 1000; 
  
  while (current_time - start_time < 1000)
    {
      if (effect & EFFECT_SLIDE)
        view->window->MoveTo 
          (view->window,
           start_x + sign_x * (view->width  * (current_time - start_time)) / 1000,
           start_y + sign_y * (view->height * (current_time - start_time)) / 1000);
      
      if (effect & EFFECT_BLEND)
        view->window->SetOpacity 
          (view->window, 
           start_opacity + sign_opacity * (current_time - start_time) / 4);
      
      gettimeofday (&tv, NULL);
      current_time = tv.tv_sec * 1000 + tv.tv_usec / 1000; 
    }
}

static void
slide_view_stop (SlideView *view)
{
  GList *list;

  g_return_if_fail (IS_SLIDE_VIEW (view));

  for (list = view->slide->items; list; list = g_list_next (list))
    {
      if (IS_VIDEO (list->data))
        video_stop ((Video *) list->data);
    }   
}

static IDirectFBWindow *
slide_view_window_create (gint *width,
                          gint *height)
{
  IDirectFBDisplayLayer *layer;
  IDirectFBWindow       *window;
  DFBDisplayLayerConfig  dlc;
  DFBWindowDescription   wdsc;
 
  dfb->GetDisplayLayer (dfb, DLID_PRIMARY, &layer);
  layer->GetConfiguration (layer, &dlc);
  
  wdsc.flags  = DWDESC_WIDTH | DWDESC_HEIGHT | DWDESC_POSX | DWDESC_POSY;
  wdsc.posx   = 0;
  wdsc.posy   = 0;
  wdsc.width  = dlc.width;
  wdsc.height = dlc.height;

  if (width)
    *width = wdsc.width;
  if (height)
    *height = wdsc.height;

  layer->CreateWindow (layer, &wdsc, &window);


  return window;
}

static void
slide_view_render_slide (SlideView *view,
                         Context   *context)
{
  IDirectFBSurface *surface;
  DFBRectangle      rect; 
  Slide *slide;
  GList *list;
  gint   y;

  slide  = view->slide;

  view->window->GetSurface (view->window, &surface);
  context_push_surface (context, surface);

  context_push (context, ITEM (slide));

  view->effect = context_get_effect (context);

  render_bg (context);
  render_bgimage (context);

  context_get_rect (context, &rect);

  if (rect.x != 0 || rect.y != 0 ||
      rect.w != context->w || rect.h != context->h)
    {
      context->surface->GetSubSurface (context->surface, 
                                       &rect, &view->subsurface);
      if (view->subsurface)
        context_push_surface (context, view->subsurface);    
    }

  y = 0;
  for (list = slide->items; list; list = g_list_next (list))
    {
      if (IS_IMAGE (list->data))
        render_image (context, (Image *) list->data);
      else if (IS_TEXT (list->data))
        render_text (context, (Text *) list->data, &y);
    }

  if (view->subsurface)
    context_pop_surface (context);

  context_pop (context, ITEM (slide));
  context_pop_surface (context);
}
