/* DirectFB presentation app
 *
 * Copyright (C) 2001  convergence integrated media
 * Authors: Sven Neumann <sven@convergence.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <directfb.h>
#include <glib-object.h>

#include "apptypes.h"

#include "context.h"
#include "dfbpoint.h"
#include "slide.h"
#include "slide-view.h"
#include "slides.h"
#include "slides-view.h"


static void  slides_view_class_init (SlidesViewClass *klass);
static void  slides_view_init       (SlidesView      *view);
static void  slides_view_finalize   (GObject         *object);


static GObjectClass *parent_class = NULL;


GType
slides_view_get_type (void)
{
  static GType slides_view_type = 0;

  if (!slides_view_type)
    {
      static const GTypeInfo slides_view_info =
      {
        sizeof (SlidesViewClass),
	(GBaseInitFunc) NULL,
	(GBaseFinalizeFunc) NULL,
	(GClassInitFunc) slides_view_class_init,
	NULL,		/* class_finalize */
	NULL,		/* class_data     */
	sizeof (SlidesView),
	0,              /* n_preallocs    */
	(GInstanceInitFunc) slides_view_init,
      };

      slides_view_type = 
        g_type_register_static (G_TYPE_OBJECT, 
                                "SlidesView", &slides_view_info, 0);
    }
  
  return slides_view_type;
}

static void
slides_view_class_init (SlidesViewClass *klass)
{
  GObjectClass *object_class;

  object_class = G_OBJECT_CLASS (klass);
  parent_class = g_type_class_peek_parent (klass);

  object_class->finalize = slides_view_finalize;
}

static void
slides_view_init (SlidesView *view)
{
  view->slides  = NULL;
  view->context = context_new ();

  view->current_slide = NULL;

  view->current  = NULL;
  view->previous = NULL;
  view->next     = NULL;
}

SlidesView *
slides_view_new (Slides *slides)
{
  IDirectFBDisplayLayer *layer;
  SlidesView *view;
  guchar     *color;

  g_return_val_if_fail (IS_SLIDES (slides), NULL);

  view = g_object_new (TYPE_SLIDES_VIEW, NULL);

  g_object_ref (slides);
  item_realize (ITEM (slides));
  context_push (view->context, ITEM (slides));

  view->slides = slides;

  if (slides->screen_width > 0 && slides->screen_height > 0 &&
      slides->bpp > 0)
    {
      g_print ("Trying to set video mode to %dx%d @ %dbpp ... ", 
               slides->screen_width, 
               slides->screen_height, 
               slides->bpp);

      /*  FIXME: workaround, should be fixed in DirectFB  */
      dfb->SetCooperativeLevel (dfb, DFSCL_FULLSCREEN);

      if (dfb->SetVideoMode (dfb,
                             slides->screen_width, 
                             slides->screen_height, 
                             slides->bpp) == DFB_OK)
        g_print ("OK\n");
      else
        g_print ("failed\n");

      /*  FIXME: workaround, should be fixed in DirectFB  */
      dfb->SetCooperativeLevel (dfb, DFSCL_NORMAL);
    }

  dfb->GetDisplayLayer (dfb, DLID_PRIMARY, &layer);
  layer->SetCooperativeLevel (layer, DLSCL_ADMINISTRATIVE);
  layer->EnableCursor (layer, FALSE);

  color = context_get_bgcolor (view->context);
  layer->SetBackgroundColor (layer, 
                             color[0], color[1], color[2], color[3]);
  layer->SetBackgroundMode (layer, DLBM_COLOR);

  if (slides->slides)
    {
      view->next = slide_view_new (SLIDE (slides->slides->data),
                                   view->context);
    }

  return view;
}

static void
slides_view_finalize (GObject *object)
{
  SlidesView *view = SLIDES_VIEW (object);

  if (view->slides)
    {
      context_pop (view->context, ITEM (view->slides));
      item_unrealize (ITEM (view->slides));
      g_object_unref (view->slides);
    }

  g_object_unref (view->context);

  if (view->current)
    g_object_unref (view->current);
  if (view->next)
    g_object_unref (view->next);
  if (view->previous)
    g_object_unref (view->previous);

  if (parent_class->finalize)
    parent_class->finalize (object);
}

gboolean
slides_view_first_slide (SlidesView *view)
{
  SlideView *first;
  GList     *next;

  g_return_val_if_fail (IS_SLIDES_VIEW (view), FALSE);

  if (!view->slides)
    return FALSE;

  if (view->current_slide &&
      view->current_slide == view->slides->slides)
    return TRUE;

  if (view->slides->slides == NULL)
    return FALSE;

  view->current_slide = view->slides->slides;

  if (view->previous)
    g_object_unref (view->previous);
  view->previous = NULL;

  first = slide_view_new (SLIDE (view->current_slide->data), view->context);
  if (view->current)
    slide_view_switch (view->current, first, BACKWARD);
  else
    slide_view_switch (NULL, first, FORWARD);

  if (view->current)
    g_object_unref (view->current);    
  view->current = first;

  if (view->next)
    g_object_unref (view->next);
  
  next = g_list_next (view->current_slide);
  view->next = 
    next ? slide_view_new (SLIDE (next->data), view->context) : NULL;

  return TRUE;
}
gboolean
slides_view_next_slide (SlidesView *view)
{
  GList *next;

  g_return_val_if_fail (IS_SLIDES_VIEW (view), FALSE);

  if (!view->slides)
    return FALSE;

  /*  special case the first slide  */
  if (! view->current_slide)
    return slides_view_first_slide (view);

  if (g_list_next (view->current_slide) == NULL)
    return FALSE;

  view->current_slide = g_list_next (view->current_slide);

  if (view->previous)
    g_object_unref (view->previous);

  view->previous = view->current;
  view->current  = view->next;

  slide_view_switch (view->previous, view->current, FORWARD);

  next = g_list_next (view->current_slide);
  view->next = 
    next ? slide_view_new (SLIDE (next->data), view->context) : NULL;

  return TRUE;
}

gboolean
slides_view_previous_slide (SlidesView *view)
{
  GList *prev;

  g_return_val_if_fail (IS_SLIDES_VIEW (view), FALSE);

  if (!view->slides)
    return FALSE;

  if (g_list_previous (view->current_slide) == NULL)
    return FALSE;

  view->current_slide = g_list_previous (view->current_slide);

  if (view->next)
    g_object_unref (view->next);

  view->next    = view->current;
  view->current = view->previous;
  
  slide_view_switch (view->next, view->current, BACKWARD);

  prev = g_list_previous (view->current_slide);
  view->previous = 
    prev ? slide_view_new (SLIDE (prev->data), view->context) : NULL;

  return TRUE;
}

Slide *
slides_view_get_current_slide (SlidesView *view)
{
  g_return_val_if_fail (IS_SLIDES_VIEW (view), NULL);
  
  if (view->current_slide)
    return SLIDE (view->current_slide->data);
  else
    return NULL;
}
